#include "main_frame.h"

#define RESOURCE_PATH "/usr/share/armadillo-demo-experience/resources/led_sw_tester"

MainFrame::MainFrame()
	: wxFrame(nullptr, wxID_ANY, "LED VIEW", wxPoint(0, 0), wxSize(800, 480))
{
	auto menuBar = new wxMenuBar;
	auto menuFile = new wxMenu;
	menuFile->Append(wxID_EXIT, wxT("Quit"));
	menuBar->Append(menuFile, wxT("&File"));
	SetMenuBar(menuBar);

	wxImage image = wxImage(wxT(RESOURCE_PATH"/ag4_base.png"), wxT("image/png"));
	wxBitmap board_bitmap(image);
	image = wxImage(wxT(RESOURCE_PATH"/ag4_clear.png"), wxT("image/png"));
	wxBitmap led_bitmap(image);
	wxBitmap sw_bitmap(image);

	this->panel = new wxPanel(this, wxID_ANY);
	this->image_panel = new wxPanel(this->panel, wxID_ANY);
	this->board_canvas = new wxStaticBitmap(this->image_panel, wxID_ANY, board_bitmap);
	this->led_canvas = new wxStaticBitmap(this->image_panel, wxID_ANY, led_bitmap);
	this->sw_canvas = new wxStaticBitmap(this->image_panel, wxID_ANY, sw_bitmap);

	this->button_panel = new wxPanel(this->panel, wxID_ANY);
	this->led_button = new wxButton(this->button_panel, LED_ID, wxT("LED"));

	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
	sizer->AddStretchSpacer();
	sizer->Add(this->image_panel, 4, wxALIGN_CENTER | wxALL, 5);
	sizer->Add(this->button_panel, 1, wxEXPAND | wxALL);
	this->panel->SetSizer(sizer);

	wxBoxSizer *btn_sizer = new wxBoxSizer(wxHORIZONTAL);
	btn_sizer->Add(this->led_button, 0, wxRIGHT, 10);
	wxBoxSizer *btn_base_sizer = new wxBoxSizer(wxVERTICAL);
	btn_base_sizer->Add(btn_sizer, 0, wxALIGN_CENTER);
	this->button_panel->SetSizer(btn_base_sizer);

	this->panel->Layout();

	Bind(wxEVT_CLOSE_WINDOW, &MainFrame::OnClose, this);
	Bind(wxEVT_MENU, [this](wxCommandEvent&) { Close(true); }, wxID_EXIT);

	this->led_controller = LEDController();
	this->led_images[0] = RESOURCE_PATH"/ag4_clear.png";
	this->led_images[1] = RESOURCE_PATH"/ag4_led_on.png";
	this->update_led_image();
}

MainFrame::~MainFrame()
{
}

void MainFrame::OnClose(wxCloseEvent& event)
{
	Destroy();
}

void MainFrame::button_pressed(wxCommandEvent &event)
{
	int status = this->led_controller.status("led1");
	if (status) {
		this->led_controller.control("led1", 0);
	} else {
		this->led_controller.control("led1", 1);
	}
	this->update_led_image();
}

void MainFrame::enter_key_pressed()
{
	wxImage image = wxImage(wxT(RESOURCE_PATH"/ag4_sw_on.png"), wxT("image/png"));
	wxBitmap sw_bitmap(image);
	this->sw_canvas->SetBitmap(sw_bitmap);
}

void MainFrame::enter_key_released()
{
	wxImage image = wxImage(wxT(RESOURCE_PATH"/ag4_clear.png"), wxT("image/png"));
	wxBitmap sw_bitmap(image);
	this->sw_canvas->SetBitmap(sw_bitmap);
}

void MainFrame::update_led_image()
{
	int status = this->led_controller.status("led1");
	std::string image_file = this->led_images[status];
	wxImage image = wxImage(image_file.c_str(), wxT("image/png"));
	wxBitmap led_bitmap(image);
	this->led_canvas->SetBitmap(led_bitmap);
}

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
	EVT_BUTTON(MainFrame::LED_ID, MainFrame::button_pressed)
END_EVENT_TABLE()
