#include "main_frame.h"

#define START_BUTTON_ID			100
#define STOP_BUTTON_ID			101
#define VIDEO_DEV_CHOICE_ID		200
#define CMD_PROCESS_ID			300

#define GST_CMD	"gst-launch-1.0 -e v4l2src device=%s ! video/x-raw,width=640,height=480,framerate=30/1 \
! tee name=t ! queue ! vpuenc_h264 ! h264parse ! queue ! mux. pulsesrc \
! audio/x-raw,rate=44100,channels=2 ! lamemp3enc ! queue ! mux. qtmux name=mux \
! filesink location=./output.mp4 t. ! queue ! waylandsink window-width=640 window_height=480"

MainFrame::MainFrame()
	: wxFrame(nullptr, wxID_ANY, "VIDEO REC TESTER VIEW", wxPoint(0, 0), wxSize(800, 600))
{
	auto menuBar = new wxMenuBar;
	auto menuFile = new wxMenu;
	menuFile->Append(wxID_EXIT, wxT("Quit"));
	menuBar->Append(menuFile, wxT("&File"));
	SetMenuBar(menuBar);

	this->panel = new wxPanel(this, wxID_ANY);
	this->setting_panel = new wxPanel(this->panel, wxID_ANY);
	this->button_panel = new wxPanel(this->panel, wxID_ANY);

	this->start_button = new wxButton(this->button_panel, START_BUTTON_ID, wxT("Start"));
	this->stop_button = new wxButton(this->button_panel, STOP_BUTTON_ID, wxT("Stop"));

	wxStaticText *video_dev_text = new wxStaticText(this->setting_panel, wxID_ANY, wxT("Video device"));
	this->video_dev_choice = new wxChoice(this->setting_panel, VIDEO_DEV_CHOICE_ID,
													wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_SORT);
	wxFileSystem *fs = new wxFileSystem();
	wxString dev = fs->FindFirst("/dev/v4l/by-id/*");
	while (!dev.empty()) {
		this->video_dev_choice->Append(wxFileSystem::URLToFileName(dev).GetFullPath()); // remove "file:"
		dev = fs->FindNext();
	}
	this->video_dev_choice->SetSelection(0);

	wxStaticText *gst_cmd_txt = new wxStaticText(this->setting_panel, wxID_ANY, wxT("Execute command line"));
	wxString gst_cmd = wxT("");
	if (this->video_dev_choice->GetCount() > 0)
		gst_cmd = wxString::Format(GST_CMD, this->video_dev_choice->GetString(0));
	this->cmd_text_ctrl = new wxTextCtrl(this->setting_panel, wxID_ANY, gst_cmd,
										wxDefaultPosition, wxSize(600, 200), wxTE_MULTILINE);

	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
	sizer->AddStretchSpacer();
	sizer->Add(this->setting_panel, 4, wxALIGN_CENTER | wxALL, 5);
	sizer->Add(this->button_panel, 1, wxEXPAND | wxALL);
	this->panel->SetSizer(sizer);

	wxBoxSizer *setting_sizer = new wxBoxSizer(wxVERTICAL);
	setting_sizer->Add(video_dev_text, 0, wxBOTTOM, 0);
	setting_sizer->Add(this->video_dev_choice, 0, wxBOTTOM, 10);
	setting_sizer->Add(gst_cmd_txt, 0, wxBOTTOM, 0);
	setting_sizer->Add(this->cmd_text_ctrl, 0, wxBOTTOM, 0);
	wxBoxSizer *setting_base_sizer = new wxBoxSizer(wxVERTICAL);
	setting_base_sizer->Add(setting_sizer, 0, wxALIGN_CENTER);
	this->setting_panel->SetSizer(setting_base_sizer);

	wxBoxSizer *btn_sizer = new wxBoxSizer(wxHORIZONTAL);
	btn_sizer->Add(this->start_button, 0, wxRIGHT, 10);
	btn_sizer->Add(this->stop_button, 0, wxRIGHT, 10);
	wxBoxSizer *btn_base_sizer = new wxBoxSizer(wxVERTICAL);
	btn_base_sizer->Add(btn_sizer, 0, wxALIGN_CENTER);
	this->button_panel->SetSizer(btn_base_sizer);

	this->panel->Layout();
	this->cmd_process = new wxProcess(this, wxID_ANY);
	this->gst_pid = 0;
	Bind(wxEVT_CLOSE_WINDOW, &MainFrame::OnClose, this);
	Bind(wxEVT_MENU, [this](wxCommandEvent&) { Close(true); }, wxID_EXIT);
}

MainFrame::~MainFrame()
{
}

void MainFrame::OnClose(wxCloseEvent& event)
{
	if (this->gst_pid > 0) {
		wxKill(this->gst_pid, wxSIGINT);
		this->gst_pid = 0;
	}
	Destroy();
}

void MainFrame::button_pressed(wxCommandEvent &event)
{
	if (event.GetId() == START_BUTTON_ID) {
		wxString gst_cmd = this->cmd_text_ctrl->GetValue();
		this->gst_pid = wxExecute(gst_cmd, wxEXEC_ASYNC, this->cmd_process);
	} else {
		if (this->gst_pid > 0) {
			wxKill(this->gst_pid, wxSIGINT);
			this->gst_pid = 0;
		}
	}
}

void MainFrame::choice_selected(wxCommandEvent &event)
{
	wxString dev = event.GetString();
	wxString gst_cmd = wxString::Format(GST_CMD, dev);
	this->cmd_text_ctrl->SetValue(gst_cmd);
}

void MainFrame::end_process(wxProcessEvent &event)
{
	int exit_code = event.GetExitCode();
	if (exit_code != 0)
		wxLogError("command execution failure. status(%d)", exit_code);
}

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
	EVT_BUTTON(START_BUTTON_ID, MainFrame::button_pressed)
	EVT_BUTTON(STOP_BUTTON_ID, MainFrame::button_pressed)
	EVT_CHOICE(VIDEO_DEV_CHOICE_ID, MainFrame::choice_selected)
	EVT_END_PROCESS(wxID_ANY, MainFrame::end_process)
END_EVENT_TABLE()
